import { } from '@stencil/core';
export class FrostedGlassContainer {
    constructor() {
        this.blurAmount = '5px';
        this.directions = ['top', 'left', 'right'];
        this.topOffset = 0;
        this.ticking = {};
        this.glassSelector = 'frosted-glass';
    }
    updateBackground() {
        this.requestTick('backgroundUpdate');
    }
    blurAmountChangeHandler(newValue) {
        this.updateFilter(newValue);
    }
    componentDidLoad() {
        this.glassElement = this.el.querySelector(this.glassSelector);
        this.createNewElements();
        this.addBaseStyles();
        this.initListeners();
    }
    componentDidUnload() {
        this.removeElements();
        this.removeListeners();
    }
    onResize() {
        this.requestTick('resizeUpdate');
    }
    onScroll() {
        this.latestKnownScrollY = window.scrollY;
        this.requestTick('scrollUpdate');
    }
    render() {
        return (h("slot", null));
    }
    createNewElements() {
        this.blurContainer = document.createElement('div');
        this.blurContent = document.createElement('div');
        this.blurContainer.appendChild(this.blurContent);
        document.querySelector('body').appendChild(this.blurContainer);
        this.requestTick('backgroundUpdate');
    }
    removeElements() {
        this.blurContainer.remove();
    }
    removeListeners() {
        window.removeEventListener('scroll', this.onScroll);
        window.removeEventListener('resize', this.onResize);
    }
    addBaseStyles() {
        Object.assign(this.blurContainer.style, {
            overflow: 'hidden',
            transform: 'translate3d(0, 0, 0)'
        });
        this.blurContent.style.position = 'absolute';
        this.updateFilter(this.blurAmount);
    }
    updateFilter(blurAmount) {
        this.blurContent.style.filter = `blur(${blurAmount})`;
    }
    initListeners() {
        if (this.glassSelector && this.glassElement && window.getComputedStyle(this.glassElement).position === 'fixed') {
            this.onScroll = this.onScroll.bind(this);
            window.addEventListener('scroll', this.onScroll);
        }
        this.onResize = this.onResize.bind(this);
        window.addEventListener('resize', this.onResize);
        this.onResize();
        this.onScroll();
    }
    // @ts-ignore
    resizeUpdate() {
        const elementStyle = window.getComputedStyle(this.glassElement);
        this.topOffset = parseInt(elementStyle.top, 10);
        ['position', 'height', ...this.directions].forEach((item) => {
            this.blurContainer.style[item] = elementStyle[item];
        });
        this.directions.forEach((item) => { this.blurContent.style[item] = `-${elementStyle[item]}`; });
        this.blurContent.style.width = window.getComputedStyle(this.el.firstElementChild).width;
        this.scrollUpdate();
        this.ticking.resizeUpdate = false;
    }
    scrollUpdate() {
        const scrollOffset = this.latestKnownScrollY + this.topOffset;
        this.blurContent.style.top = `-${scrollOffset}px`;
        this.ticking.scrollUpdate = false;
    }
    requestTick(functionName) {
        if (!this.ticking[functionName]) {
            requestAnimationFrame(this[functionName].bind(this));
        }
        this.ticking[functionName] = true;
    }
    // @ts-ignore
    backgroundUpdate() {
        this.blurContent.innerHTML = this.el.innerHTML;
        const glassClone = this.blurContent.querySelector(this.glassSelector);
        if (glassClone) {
            glassClone.remove();
        }
        this.ticking.backgroundUpdate = false;
    }
}
